'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.toPublishOptions = toPublishOptions;
exports.publish = publish;

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _options = require('../utils/options');

var options = _interopRequireWildcard(_options);

var _errors = require('../utils/errors');

var _logger = require('../utils/logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('../utils/messages');

var messages = _interopRequireWildcard(_messages);

var _locks = require('../utils/locks');

var locks = _interopRequireWildcard(_locks);

var _npm = require('../utils/npm');

var npm = _interopRequireWildcard(_npm);

var _Project = require('../Project');

var _Project2 = _interopRequireDefault(_Project);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toPublishOptions(args, flags) {
  return {
    cwd: options.string(flags.cwd, 'cwd'),
    access: options.string(flags.access, 'access')
  };
}

async function getUnpublishedPackages(packages) {
  const results = await _promise2.default.all(packages.map(async pkg => {
    let config = pkg.config;
    let response = await npm.infoAllow404(config.getName());

    return {
      name: config.getName(),
      localVersion: config.getVersion(),
      isPublished: response.published,
      publishedVersion: response.pkgInfo.version || ''
    };
  }));

  return results.filter(result => {
    // only publish if our version is higher than the one on npm
    return !result.isPublished || _semver2.default.gt(result.localVersion, result.publishedVersion);
  });
}

async function publish(opts) {
  const cwd = opts.cwd || process.cwd();
  const project = await _Project2.default.init(cwd);
  const workspaces = await project.getWorkspaces();
  const packages = workspaces.map(workspace => workspace.pkg).filter(pkg => !pkg.config.getPrivate());

  try {
    // TODO: Re-enable once locking issues are sorted out
    // await locks.lock(packages);
    const unpublishedPackages = await getUnpublishedPackages(packages);
    const isUnpublished = workspace => unpublishedPackages.some(pkg => workspace.pkg.config.getName() === pkg.name);
    const unpublishedWorkspaces = workspaces.filter(isUnpublished);

    if (unpublishedPackages.length === 0) {
      logger.warn(messages.noUnpublishedPackagesToPublish());
    }

    await _Project2.default.runWorkspaceTasks(unpublishedWorkspaces, async workspace => {
      const name = workspace.pkg.config.getName();
      const version = workspace.pkg.config.getVersion();
      logger.info(messages.publishingPackage(name, version));

      await npm.publish(name, { cwd: workspace.pkg.dir, access: opts.access });
    });

    // TODO: Re-enable once locking issues are sorted out
    // await locks.unlock(packages);
  } catch (err) {
    logger.error(err.message);
    throw new _errors.BoltError('Failed to publish');
  }
}