'use strict';

exports.__esModule = true;
exports.toWorkspaceRunOptions = toWorkspaceRunOptions;
exports.workspaceRun = workspaceRun;

var _Project = require('../../Project');

var _Project2 = _interopRequireDefault(_Project);

var _options = require('../../utils/options');

var options = _interopRequireWildcard(_options);

var _yarn = require('../../utils/yarn');

var yarn = _interopRequireWildcard(_yarn);

var _errors = require('../../utils/errors');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toWorkspaceRunOptions(args, flags) {
  let [workspaceName, script, ...scriptArgs] = args;
  return {
    cwd: options.string(flags.cwd, 'cwd'),
    workspaceName,
    script,
    scriptArgs
  };
}

async function workspaceRun(opts) {
  let cwd = opts.cwd || process.cwd();
  let project = await _Project2.default.init(cwd);
  let workspaces = await project.getWorkspaces();
  let workspace = await project.getWorkspaceByName(workspaces, opts.workspaceName);

  if (!workspace) {
    throw new _errors.BoltError(`Could not find a workspace named "${opts.workspaceName}" from "${cwd}"`);
  }

  let validScript = await yarn.getScript(workspace.pkg, opts.script);

  if (!validScript) {
    throw new _errors.BoltError(`Package at "${workspace.pkg.dir}" does not have a script named "${opts.script}"`);
  }
  await yarn.run(workspace.pkg, opts.script, opts.scriptArgs);
}